package com.example.ellegadodepintia.ruleta

import android.animation.ObjectAnimator
import android.animation.ValueAnimator
import android.os.Bundle
import android.os.Handler
import android.os.Looper
import android.view.ViewGroup
import android.view.animation.AccelerateDecelerateInterpolator
import android.view.animation.AnimationUtils
import android.view.animation.RotateAnimation
import android.widget.ImageView
import android.widget.TextView
import androidx.appcompat.app.AppCompatActivity
import androidx.core.animation.doOnEnd
import com.example.ellegadodepintia.R
import kotlin.random.Random
import com.example.ellegadodepintia.soundManager.SoundManager

class Ruleta : AppCompatActivity() {
    private lateinit var ruletaImage: ImageView
    private lateinit var puntero: ImageView
    private lateinit var titulo: ImageView
    private lateinit var multiplicadorResultado: TextView
    private lateinit var puntuacion: TextView
    private lateinit var soundManager: SoundManager

    private val segmentos = arrayOf("x4", "x1", "x2", "x1", "x2", "x1", "x2", "x1")
    private var ultimaRotacion = 0f
    private var puntuacionActual = 33

    // Configura las vistas iniciales de la actividad y establece acciones iniciales
    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        soundManager = SoundManager(this)
        soundManager.playSoundLoop(R.raw.sound_casino, 80)
        setContentView(R.layout.activity_ruleta)
        inicializarVistas()
        animarTitulo()
        configurarClickRuleta()

    }

    // Inicializa las vistas del layout asociadas a la ruleta
    private fun inicializarVistas() {
        ruletaImage = findViewById(R.id.ruletaImage)
        puntero = findViewById(R.id.ruletaPuntero)
        multiplicadorResultado = findViewById(R.id.multiplicadorResultado)
        puntuacion = findViewById(R.id.puntuacion)
        titulo = findViewById(R.id.ruletaTitulo)
    }

    // Aplica una animación al título de la ruleta
    private fun animarTitulo() {
        val animation = AnimationUtils.loadAnimation(this, R.anim.animatitu)
        titulo.startAnimation(animation)
    }

    // Configura el evento de clic para que la ruleta gire cuando se presiona
    private fun configurarClickRuleta() {
        ruletaImage.setOnClickListener {
            soundManager.playSound(R.raw.sound_ruleta,100)
            girarRuleta()
        }
    }

    // Realiza la animación de giro de la ruleta y determina el ángulo de rotación
    private fun girarRuleta() {
        ruletaImage.rotation = -24f

        val randomAngleMultiplier = Random.nextInt(0, segmentos.size)
        val extraRotation = randomAngleMultiplier * 45
        val rotacionTotal = ultimaRotacion + 1800 + extraRotation

        val rotateAnim = RotateAnimation(
            ultimaRotacion, rotacionTotal,
            RotateAnimation.RELATIVE_TO_SELF, 0.5f,
            RotateAnimation.RELATIVE_TO_SELF, 0.5f
        ).apply {
            duration = 4000
            fillAfter = true
            interpolator = AccelerateDecelerateInterpolator()
        }
        ruletaImage.startAnimation(rotateAnim)
        ultimaRotacion = rotacionTotal

        val handler = Handler(Looper.getMainLooper())
        handler.postDelayed({ manejarResultadoGiro(rotacionTotal) }, 4000)
    }

    // Maneja el resultado tras finalizar el giro de la ruleta, calcula el multiplicador y actualiza la puntuación
    private fun manejarResultadoGiro(rotacionTotal: Float) {
        val anguloSegmento = 360 / segmentos.size
        val anguloFinal = rotacionTotal % 360
        val index = (anguloFinal / anguloSegmento).toInt()
        val multiplicador = segmentos[index]
        val multiplicadorValor = multiplicador.drop(1).toInt()
        val nuevaPuntuacion = puntuacionActual * multiplicadorValor
        animarPuntuacion(puntuacionActual, nuevaPuntuacion)
        puntuacionActual = nuevaPuntuacion

        if (multiplicador == "x4") {
            soundManager.playSound(R.raw.sound_big_win,100)
            lluviaMonedas()
        }
    }

    // Realiza una animación de conteo que muestra la puntuación cambiando de un valor inicial a un valor final
    private fun animarPuntuacion(inicio: Int, fin: Int) {
        ValueAnimator.ofInt(inicio, fin).apply {
            duration = 3000
            addUpdateListener { animation ->
                val valorActual = animation.animatedValue as Int
                puntuacion.text = "Puntuación: $valorActual"
            }
            start()
        }
    }

    // Inicia un efecto de lluvia de monedas cuando se obtiene un 'x4'
    private fun lluviaMonedas() {
        val screenWidth = resources.displayMetrics.widthPixels
        val screenHeight = resources.displayMetrics.heightPixels
        val rainDuration = 2500L
        val handler = Handler(Looper.getMainLooper())
        val startTime = System.currentTimeMillis()
        val runnable = object : Runnable {
            override fun run() {
                val elapsedTime = System.currentTimeMillis() - startTime
                if (elapsedTime < rainDuration) {
                    monedaCaer(screenWidth, screenHeight)
                    handler.postDelayed(this, 10)
                }
            }
        }
        handler.post(runnable)
    }

    // Crea una moneda animada que cae desde la parte superior de la pantalla
    private fun monedaCaer(screenWidth: Int, screenHeight: Int) {
        val coin = ImageView(this).apply {
            setImageResource(R.drawable.asi_ruleta_moneda)
            layoutParams = ViewGroup.LayoutParams(100, 100)
            x = Random.nextInt(0, screenWidth - 100).toFloat()
            y = -200f
        }
        addContentView(coin, coin.layoutParams)
        ObjectAnimator.ofFloat(coin, "translationY", screenHeight.toFloat() + 500).apply {
            duration = Random.nextLong(2000, 3000)
            start()
            doOnEnd { (coin.parent as? ViewGroup)?.removeView(coin) }
        }
    }
    override fun onPause() {
        super.onPause()
        soundManager.stopSound()
    }
}
